// this is a very basic template. use it to start writing your own effects.
// if you want effects with lighting start from one of the GouraudXXXX or PhongXXXX effects

// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;
float4x4 tWVP: WORLDVIEWPROJECTION;
float brightness;
float contrast;
bool bw;
bool invert;
bool dilate;
float dFactor;
bool erode;
float eFactor;
float4x4 tCol;
float texWidth;
bool pointFilter;

//texture
texture Tex <string uiname="Texture";>;

sampler Samp = sampler_state
{
       Texture   = (Tex);
       MipFilter = LINEAR;
       MinFilter = LINEAR;
       MagFilter = LINEAR;

};

sampler SampPoint = sampler_state
{
       Texture   = (Tex);
       MipFilter = POINT;
       MinFilter = POINT;
       MagFilter = POINT;

};

//texture transformation marked with semantic TEXTUREMATRIX to achieve symmetric transformations
float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos  : POSITION;
    float2 TexCd : TEXCOORD0;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------
vs2ps VS(
    float4 PosO  : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
    Out.Pos = mul(PosO, tWVP);
    
    //transform texturecoordinates
    Out.TexCd = mul(TexCd, tTex);

    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 PS(vs2ps In): COLOR
{
    float4 col;
    if (!pointFilter){
       col = tex2D(Samp, In.TexCd);
    }
    else col = tex2D(SampPoint, In.TexCd);


    float2 neighbours[8] = {
        -1, -1,
        0, -1,
        1, -1,
        -1, 0,
        1, 0,
        -1, 1,
        0, 1,
        1, 1,
         };

    // dilate
    
    if (dilate) {
    for (int i = 0; i < 8; i++) {
        float4 tmpSample;
        if (!pointFilter){
           tmpSample = tex2D(Samp, In.TexCd + (1/texWidth*dFactor) * neighbours[i]);
        }
        else tmpSample = tex2D(SampPoint, In.TexCd + (1/texWidth*dFactor) * neighbours[i]);
        
        col = min(col, tmpSample);
    }
    }
    
    // erode
    if (erode) {
    for (int i = 0; i < 8; i++) {
        float4 tmpSample;
         if (!pointFilter){
           tmpSample = tex2D(Samp, In.TexCd + (1/texWidth*eFactor) * neighbours[i]);
        }
        else tmpSample = tex2D(SampPoint, In.TexCd + (1/texWidth*eFactor) * neighbours[i]);
        col = max(col, tmpSample);
    }
    }

    // invert
    if (invert) {
    col.r = 1 - col.r;
    col.g = 1 - col.g;
    col.b = 1 - col.b;
    }

    // contrast & brightness
    col -= 0.5;
    col *= contrast;
    col += 0.5;
    col += brightness;

    // turn to b&w
    if (bw) {
       col = .2125 * col.r + .7154 * col.r + .0721 *  col.b;
    }
 
    return mul(tCol,col);
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique TSimpleShader
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_3_0 PS();
    }
}

technique TFixedFunction
{
    pass P0
    {
        //transforms
        WorldTransform[0]   = (tW);
        ViewTransform       = (tV);
        ProjectionTransform = (tP);

        //texturing
        Sampler[0] = (Samp);
        TextureTransform[0] = (tTex);
        TexCoordIndex[0] = 0;
        TextureTransformFlags[0] = COUNT2;
        //Wrap0 = U;  // useful when mesh is round like a sphere
        
        Lighting       = FALSE;

        //shaders
        VertexShader = NULL;
        PixelShader  = NULL;
    }
}
